@@ -45,7 +45,7 @@ class TimeRecurrenceParser(object):
     """
 
     RECURRENCE_REGEXES = [
-        re.compile(r"^R(?P<reps>\d+)/(?P<start>[^P][^/]*)/(?P<end>[^P].*)$"),
+        re.compile(r"^R(?P<reps>\d+)?/(?P<start>[^P][^/]*)/(?P<end>[^P].*)$"),
         re.compile(r"^R(?P<reps>\d+)?/(?P<start>[^P][^/]*)/(?P<intv>P.+)$"),
         re.compile(r"^R(?P<reps>\d+)?/(?P<intv>P.+)/(?P<end>[^P].*)$")]
 
@@ -131,7 +131,7 @@ class TimePointParser(object):
                  assumed_time_zone=None,
                  default_to_unknown_time_zone=False,
                  dump_format=None):
-        self.expanded_year_digits = num_expanded_year_digits
+        self.num_expanded_year_digits = num_expanded_year_digits
         self.allow_truncated = allow_truncated
         self.allow_only_basic = allow_only_basic
         self.assumed_time_zone = assumed_time_zone
@@ -191,7 +191,7 @@ class TimePointParser(object):
         """Construct regular expressions for the date."""
         for expr_regex, substitute, _, _ in (
                 parser_spec.get_date_translate_info(
-                    self.expanded_year_digits)):
+                    self.num_expanded_year_digits)):
             expression = re.sub(expr_regex, substitute, expression)
         expression = "^" + expression + "$"
         return expression
@@ -276,7 +276,8 @@ class TimePointParser(object):
             year += 100 * int(date_info.pop("century", 0))
             expanded_year = date_info.pop("expanded_year", 0)
             if expanded_year:
-                date_info["expanded_year_digits"] = self.expanded_year_digits
+                date_info["num_expanded_year_digits"] = (
+                    self.num_expanded_year_digits)
             year += 10000 * int(expanded_year)
             if date_info.pop("year_sign", "+") == "-":
                 year *= -1
@@ -355,7 +356,7 @@ class TimePointParser(object):
                 info.update(translator(value))
         date_info_keys = []
         for item in parser_spec.get_date_translate_info(
-                self.expanded_year_digits):
+                self.num_expanded_year_digits):
             date_info_keys.append(item[3])
         time_info_keys = []
         for item in parser_spec.get_time_translate_info():
@@ -589,17 +590,17 @@ class DurationParser(object):
             if timepoint.get_is_week_date():
                 raise ISO8601SyntaxError("duration", expression)
             result_map = {}
-            result_map["years"] = timepoint.year
+            result_map["years"] = timepoint._year
             if timepoint.get_is_calendar_date():
-                result_map["months"] = timepoint.month_of_year
-                result_map["days"] = timepoint.day_of_month
+                result_map["months"] = timepoint._month_of_year
+                result_map["days"] = timepoint._day_of_month
             if timepoint.get_is_ordinal_date():
-                result_map["days"] = timepoint.day_of_year
-            result_map["hours"] = timepoint.hour_of_day
-            if timepoint.minute_of_hour is not None:
-                result_map["minutes"] = timepoint.minute_of_hour
-            if timepoint.second_of_minute is not None:
-                result_map["seconds"] = timepoint.second_of_minute
+                result_map["days"] = timepoint._day_of_year
+            result_map["hours"] = timepoint._hour_of_day
+            if timepoint._minute_of_hour is not None:
+                result_map["minutes"] = timepoint._minute_of_hour
+            if timepoint._second_of_minute is not None:
+                result_map["seconds"] = timepoint._second_of_minute
             return data.Duration(**result_map)
         raise ISO8601SyntaxError("duration", expression)
 
