@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-# ----------------------------------------------------------------------------
 # Copyright (C) British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
@@ -14,24 +12,21 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-# ----------------------------------------------------------------------------
 
 """This tests the ISO 8601 parsing and data model functionality."""
 
 import copy
 import datetime
 from itertools import chain
-import pytest
 import unittest
-from unittest.mock import patch, MagicMock, Mock
+import pytest
 
 from metomi.isodatetime import data
 from metomi.isodatetime import dumpers
 from metomi.isodatetime import parsers
 from metomi.isodatetime import parser_spec
-from metomi.isodatetime import timezone
 from metomi.isodatetime.exceptions import (
-    ISO8601SyntaxError, TimePointDumperBoundsError)
+    ISO8601SyntaxError, TimePointDumperBoundsError, BadInputError)
 
 
 def get_timedurationparser_tests():
@@ -136,11 +131,13 @@ def get_timepoint_dumper_tests():
               "+000044-01-04T05:01:02+00:00"),
              ("DD/MM/CCYY is a silly format", "04/01/0044 is a silly format"),
              ("ThhZ", "T05Z"),
-             ("%Y-%m-%dT%H:%M", "0044-01-04T05:01")]
+             ("%Y-%m-%dT%H:%M", "0044-01-04T05:01"),
+             ("CCYYMMDD00.T+3", "0044010400.T+3"),
+             ("T+3", "T+3")]
         ),
         (
             {"year": 500200, "month_of_year": 7, "day_of_month": 28,
-             "expanded_year_digits": 2, "hour_of_day": 0,
+             "num_expanded_year_digits": 2, "hour_of_day": 0,
              "hour_of_day_decimal": 0.4356, "time_zone_hour": -8,
              "time_zone_minute": -30},
             [("+XCCYY-MMDDThhmmZ", "+500200-0728T0856Z"),
@@ -164,10 +161,11 @@ def get_timepoint_dumper_tests():
              ("DD/MM/+XCCYY is a silly format",
               "28/07/+500200 is a silly format"),
              ("ThhmmZ", "T0856Z"),
-             ("%m-%dT%H:%M", "07-28T00:26")]
+             ("%m-%dT%H:%M", "07-28T00:26"),
+             ("+XCCYY-MM-DDThh,ii", "+500200-07-28T00,4356")]
         ),
         (
-            {"year": -56, "day_of_year": 318, "expanded_year_digits": 2,
+            {"year": -56, "day_of_year": 318, "num_expanded_year_digits": 2,
              "hour_of_day": 5, "minute_of_hour": 1, "time_zone_hour": 6},
             [("+XCCYY-MMDDThhmmZ", "-000056-1112T2301Z"),
              ("+XCCYYDDDThh:mm:ss", "-000056318T05:01:00"),
@@ -208,6 +206,13 @@ def get_timepoint_dumper_tests():
              ("CCYY-Www-DThhmm+0200", "1000-W01-1T0200+0200"),
              ("CCYY-Www-DThhmm-0200", "0999-W52-7T2200-0200"),
              ("%Y-%m-%dT%H:%M", "0999-12-30T00:00")]
+        ),
+        (
+            {"year": 2027, "month_of_year": 12, "day_of_month": 31,
+             "minute_of_hour": 59, "minute_of_hour_decimal": 0.99999999},
+            [("CCYY-MM-DDThh:mm,nnZ", "2027-12-31T00:59,999999Z"),
+             ("CCYY-MM-DDThh:mm.nnZ", "2027-12-31T00:59.999999Z"),
+             ("Thh:mm:ss,tt", "T00:59:59,999999")]
         )
     ]
 
@@ -263,44 +268,45 @@ def get_timepointparser_tests(allow_only_basic=False,
                              "day_of_month": 4},
                 "+5002000830": {"year": 500200, "month_of_year": 8,
                                 "day_of_month": 30,
-                                "expanded_year_digits": 2},
+                                "num_expanded_year_digits": 2},
                 "-0000561113": {"year": -56, "month_of_year": 11,
                                 "day_of_month": 13,
-                                "expanded_year_digits": 2},
+                                "num_expanded_year_digits": 2},
                 "-1000240210": {"year": -100024, "month_of_year": 2,
                                 "day_of_month": 10,
-                                "expanded_year_digits": 2},
+                                "num_expanded_year_digits": 2},
                 "1967056": {"year": 1967, "day_of_year": 56},
                 "+123456078": {"year": 123456, "day_of_year": 78,
-                               "expanded_year_digits": 2},
+                               "num_expanded_year_digits": 2},
                 "-004560134": {"year": -4560, "day_of_year": 134,
-                               "expanded_year_digits": 2},
+                               "num_expanded_year_digits": 2},
                 "1001W011": {"year": 1001, "week_of_year": 1,
                              "day_of_week": 1},
                 "+000001W457": {"year": 1, "week_of_year": 45,
                                 "day_of_week": 7,
-                                "expanded_year_digits": 2},
+                                "num_expanded_year_digits": 2},
                 "-010001W053": {"year": -10001, "week_of_year": 5,
-                                "day_of_week": 3, "expanded_year_digits": 2}
+                                "day_of_week": 3,
+                                "num_expanded_year_digits": 2}
             },
             "reduced": {
                 "4401-03": {"year": 4401, "month_of_year": 3},
                 "1982": {"year": 1982},
                 "19": {"year": 1900},
                 "+056789-01": {"year": 56789, "month_of_year": 1,
-                               "expanded_year_digits": 2},
+                               "num_expanded_year_digits": 2},
                 "-000001-12": {"year": -1, "month_of_year": 12,
-                               "expanded_year_digits": 2},
-                "-789123": {"year": -789123, "expanded_year_digits": 2},
-                "+450001": {"year": 450001, "expanded_year_digits": 2},
+                               "num_expanded_year_digits": 2},
+                "-789123": {"year": -789123, "num_expanded_year_digits": 2},
+                "+450001": {"year": 450001, "num_expanded_year_digits": 2},
                 # The following cannot be parsed - looks like truncated -YYMM.
-                #  "-0023": {"year": -2300, "expanded_year_digits": 2},
-                "+5678": {"year": 567800, "expanded_year_digits": 2},
+                #  "-0023": {"year": -2300, "num_expanded_year_digits": 2},
+                "+5678": {"year": 567800, "num_expanded_year_digits": 2},
                 "1765W04": {"year": 1765, "week_of_year": 4},
                 "+001765W44": {"year": 1765, "week_of_year": 44,
-                               "expanded_year_digits": 2},
+                               "num_expanded_year_digits": 2},
                 "-123321W50": {"year": -123321, "week_of_year": 50,
-                               "expanded_year_digits": 2}
+                               "num_expanded_year_digits": 2}
             },
             "truncated": {
                 "-9001": {"year": 90, "month_of_year": 1,
@@ -339,45 +345,45 @@ def get_timepointparser_tests(allow_only_basic=False,
                                "day_of_month": 4},
                 "+500200-08-30": {"year": 500200, "month_of_year": 8,
                                   "day_of_month": 30,
-                                  "expanded_year_digits": 2},
+                                  "num_expanded_year_digits": 2},
                 "-000056-11-13": {"year": -56, "month_of_year": 11,
                                   "day_of_month": 13,
-                                  "expanded_year_digits": 2},
+                                  "num_expanded_year_digits": 2},
                 "-100024-02-10": {"year": -100024, "month_of_year": 2,
                                   "day_of_month": 10,
-                                  "expanded_year_digits": 2},
+                                  "num_expanded_year_digits": 2},
                 "1967-056": {"year": 1967, "day_of_year": 56},
                 "+123456-078": {"year": 123456, "day_of_year": 78,
-                                "expanded_year_digits": 2},
+                                "num_expanded_year_digits": 2},
                 "-004560-134": {"year": -4560, "day_of_year": 134,
-                                "expanded_year_digits": 2},
+                                "num_expanded_year_digits": 2},
                 "1001-W01-1": {"year": 1001, "week_of_year": 1,
                                "day_of_week": 1},
                 "+000001-W45-7": {"year": 1, "week_of_year": 45,
                                   "day_of_week": 7,
-                                  "expanded_year_digits": 2},
+                                  "num_expanded_year_digits": 2},
                 "-010001-W05-3": {"year": -10001, "week_of_year": 5,
                                   "day_of_week": 3,
-                                  "expanded_year_digits": 2}
+                                  "num_expanded_year_digits": 2}
             },
             "reduced": {
                 "4401-03": {"year": 4401, "month_of_year": 3},
                 "1982": {"year": 1982},
                 "19": {"year": 1900},
                 "+056789-01": {"year": 56789, "month_of_year": 1,
-                               "expanded_year_digits": 2},
+                               "num_expanded_year_digits": 2},
                 "-000001-12": {"year": -1, "month_of_year": 12,
-                               "expanded_year_digits": 2},
-                "-789123": {"year": -789123, "expanded_year_digits": 2},
-                "+450001": {"year": 450001, "expanded_year_digits": 2},
+                               "num_expanded_year_digits": 2},
+                "-789123": {"year": -789123, "num_expanded_year_digits": 2},
+                "+450001": {"year": 450001, "num_expanded_year_digits": 2},
                 # The following cannot be parsed - looks like truncated -YYMM.
-                #  "-0023": {"year": -2300, "expanded_year_digits": 2},
-                "+5678": {"year": 567800, "expanded_year_digits": 2},
+                #  "-0023": {"year": -2300, "num_expanded_year_digits": 2},
+                "+5678": {"year": 567800, "num_expanded_year_digits": 2},
                 "1765-W04": {"year": 1765, "week_of_year": 4},
                 "+001765-W44": {"year": 1765, "week_of_year": 44,
-                                "expanded_year_digits": 2},
+                                "num_expanded_year_digits": 2},
                 "-123321-W50": {"year": -123321, "week_of_year": 50,
-                                "expanded_year_digits": 2}
+                                "num_expanded_year_digits": 2}
             },
             "truncated": {
                 "-9001": {"year": 90, "month_of_year": 1,
@@ -592,10 +598,7 @@ def get_timepointparser_tests(allow_only_basic=False,
 
 
 def get_truncated_property_tests():
-    """
-    Tests for largest truncated and
-    smallest missing property names
-    """
+    """Tests for largest truncated and smallest missing property names."""
     test_timepoints = {
         "-9001": {"year": 90,
                   "month_of_year": 1,
@@ -682,11 +685,18 @@ def get_truncated_property_tests():
 
 
 def get_timerecurrence_expansion_tests():
-    """Return test expansion expressions for data.TimeRecurrence."""
+    """Return test expansion expressions for data.TimeRecurrence.
+
+    If no. of repetitions is unbounded, will test the first three.
+    """
     return [
+        ("R5/2020-01-01T00:00:00Z/2020-01-05T00:00:00Z",
+         ["2020-01-01T00:00:00Z", "2020-01-05T00:00:00Z",
+          "2020-01-09T00:00:00Z", "2020-01-13T00:00:00Z",
+          "2020-01-17T00:00:00Z"]),
         ("R3/1001-W01-1T00:00:00Z/1002-W52-6T00:00:00-05:30",
-         ["1001-W01-1T00:00:00Z", "1001-W53-3T14:45:00Z",
-          "1002-W52-6T05:30:00Z"]),
+         ["1001-W01-1T00:00:00Z", "1002-W52-6T05:30:00Z",
+          "1005-W01-4T11:00:00Z"]),
         ("R3/P700D/1957-W01-1T06,5Z",
          ["1953-W10-1T06,5Z", "1955-W05-1T06,5Z", "1957-W01-1T06,5Z"]),
         ("R3/P5DT2,5S/1001-W11-1T00:30:02,5-02:00",
@@ -707,6 +717,33 @@ def get_timerecurrence_expansion_tests():
     ]
 
 
+def get_timerecurrence_comparison_tests():
+    """Yield tests for executing the '==' operator and hash() on
+    TimeRecurrences."""
+    return [
+        ("R5/2020-036T00Z/PT15M", "R5/2020-036T00Z/PT15M", True),
+        *[("R5/2020-036T00Z/PT15M", rhs, False) for rhs in (
+            "R4/2020-036T00Z/PT15M",
+            "R5/2020-036T01Z/PT15M",
+            "R5/2020-036T00Z/PT14M")],
+        # Format 3 vs 4:
+        ("R4/2020-01-01T00Z/P1D", "R4/P1D/2020-01-04T00Z", True),
+        ("R/2020-01-01T00Z/P1D", "R/P1D/2020-01-04T00Z", False),
+        # Format 1 vs 3:
+        ("R5/2020-001T00Z/2020-005T00Z", "R5/2020-001T00Z/P4D", True),
+        ("R/2020-02-07T09Z/2020-02-07T10Z", "R/2020-02-07T09Z/PT1H", True),
+        # Format 1 vs 4:
+        ("R5/2020-001T00Z/2020-005T00Z", "R5/P4D/2020-005T00Z", False),
+        ("R5/2020-001T00Z/2020-005T00Z", "R5/P4D/2020-017T00Z", True),
+        ("R/2020-02-07T09Z/2020-02-07T10Z", "R/PT1H/2020-02-07T10Z", False),
+        # Mixed stuff:
+        ("R3/2020-05-01T16Z/PT3H", "R3/2020-W18-5T18:30+02:30/PT180M", True),
+        # Start point == end point == one repetition:
+        ("R27/2020-10-08T00Z/2020-10-08T04+04",
+         "R1/2020-10-08T00Z/2020-10-08T00Z", True)
+    ]
+
+
 def get_timerecurrence_expansion_tests_for_alt_calendar(calendar_mode):
     """Return alternate calendar tests for data.TimeRecurrence."""
     if calendar_mode == "360":
@@ -826,15 +863,20 @@ def get_timerecurrence_expansion_tests_366():
 def get_timerecurrence_membership_tests():
     """Return test membership expressions for data.TimeRecurrence."""
     return [
+        ("R5/2020-01-01T00:00:00Z/2020-01-05T00:00:00Z",
+         [("2020-01-02T00:00:00Z", False),
+          ("2020-01-17T00:00:00Z", True),
+          ("2020-01-21T00:00:00Z", False)]),
         ("R3/1001-W01-1T00:00:00Z/1002-W52-6T00:00:00-05:30",
          [("1001-W01-1T00:00:00Z", True),
           ("1000-12-29T00:00:00Z", True),
           ("0901-07-08T12:45:00Z", False),
           ("1001-W01-2T00:00:00Z", False),
-          ("1001-W53-3T14:45:00Z", True),
+          ("1001-W53-3T14:45:00Z", False),
           ("1002-W52-6T05:30:00Z", True),
           ("1002-W52-6T03:30:00-02:00", True),
           ("1002-W52-6T07:30:00+02:00", True),
+          ("1005-W01-4T11:00:00Z", True),
           ("10030101T00Z", False)]),
         ("R3/P700D/1957-W01-1T06,5Z",
          [("1953-W10-1T06,5Z", True),
@@ -847,6 +889,26 @@ def get_timerecurrence_membership_tests():
     ]
 
 
+def get_timerecurrence_first_after_tests():
+    """Return test get_first_after expressions for data.TimeRecurrence"""
+    return [
+        ("R5/2020-01-01T00:00:00Z/2020-01-05T00:00:00Z",
+         [("2019-01-01T00:00:00Z", "2020-01-01T00:00:00Z"),
+          ("2020-01-02T00:00:00Z", "2020-01-05T00:00:00Z"),
+          ("2020-01-15T00:00:00Z", "2020-01-17T00:00:00Z"),
+          ("2021-01-01T00:00:00Z", "None")]),
+        ("R3/P700D/1961-W01-1T06,5Z",
+         [("1950-01-01T00:00:00Z", "1957-W10-1T06,5Z"),
+          ("1958-01-01T00:00:00Z", "1959-02-02T06:30:00Z"),
+          ("2021-01-01T00:00:00Z", "None")]),
+        ("R/2000-01-01T00:00:00Z/P1Y",
+         [("2001-01-01T00:00:00Z", "2002-01-01T00:00:00Z"),
+          ("2002-01-01T00:00:00Z", "2003-01-01T00:00:00Z")]),
+        ("R1/2000-01-01T00:00:00Z/P1Y",
+         [("2000-01-01T00:00:00Z", "None")])
+    ]
+
+
 def get_timerecurrenceparser_tests():
     """Yield tests for the time recurrence parser."""
     test_points = ["-100024-02-10T17:00:00-12:30",
@@ -870,16 +932,13 @@ def get_timerecurrenceparser_tests():
                     continue
                 duration = duration_parser.parse(duration_expr)
                 end_point = start_point + duration
-                if reps is not None:
-                    expr_1 = ("R" + reps_string + "/" + str(start_point) +
-                              "/" + str(end_point))
-                    yield expr_1, {"repetitions": reps,
-                                   "start_point": start_point,
-                                   "end_point": end_point}
+                expr_1 = ("R" + reps_string + "/" + str(start_point) +
+                          "/" + str(end_point))
+                yield expr_1, {"repetitions": reps, "start_point": start_point,
+                               "end_point": end_point}
                 expr_3 = ("R" + reps_string + "/" + str(start_point) +
                           "/" + str(duration))
-                yield expr_3, {"repetitions": reps,
-                               "start_point": start_point,
+                yield expr_3, {"repetitions": reps, "start_point": start_point,
                                "duration": duration}
                 expr_4 = ("R" + reps_string + "/" + str(duration) + "/" +
                           str(end_point))
@@ -966,46 +1025,36 @@ class TestSuite(unittest.TestCase):
 
     def test_timepoint_time_zone(self):
         """Test the time zone handling of timepoint instances."""
-        year = 2000
-        month_of_year = 1
-        day_of_month = 1
+        year, month_of_year, day_of_month = (2000, 1, 1)
         utc_offset_hours, utc_offset_minutes = (
             get_local_time_zone_hours_minutes()
         )
         for hour_of_day in range(24):
             for minute_of_hour in [0, 30]:
+                point = data.TimePoint(year=year, month_of_year=month_of_year,
+                                       day_of_month=day_of_month,
+                                       hour_of_day=hour_of_day,
+                                       minute_of_hour=minute_of_hour)
                 test_dates = [
-                    data.TimePoint(
-                        year=year,
-                        month_of_year=month_of_year,
-                        day_of_month=day_of_month,
-                        hour_of_day=hour_of_day,
-                        minute_of_hour=minute_of_hour
-                    )
+                    point.to_utc(),
+                    point.to_local_time_zone(),
+                    point.to_time_zone(data.TimeZone(hours=-13, minutes=-45)),
+                    point.to_time_zone(data.TimeZone(hours=8, minutes=30))
                 ]
-                test_dates.append(test_dates[0].copy())
-                test_dates.append(test_dates[0].copy())
-                test_dates.append(test_dates[0].copy())
-                test_dates[0].set_time_zone_to_utc()
                 self.assertEqual(test_dates[0].time_zone.hours, 0,
                                  test_dates[0])
                 self.assertEqual(test_dates[0].time_zone.minutes, 0,
                                  test_dates[0])
-                test_dates[1].set_time_zone_to_local()
+
                 self.assertEqual(test_dates[1].time_zone.hours,
                                  utc_offset_hours, test_dates[1])
-
                 self.assertEqual(test_dates[1].time_zone.minutes,
                                  utc_offset_minutes, test_dates[1])
-                test_dates[2].set_time_zone(
-                    data.TimeZone(hours=-13, minutes=-45))
 
-                test_dates[3].set_time_zone(
-                    data.TimeZone(hours=8, minutes=30))
                 for i_test_date in list(test_dates):
                     i_test_date_str = str(i_test_date)
-                    date_no_tz = i_test_date.copy()
-                    date_no_tz.time_zone = data.TimeZone(hours=0, minutes=0)
+                    date_no_tz = i_test_date._copy()
+                    date_no_tz._time_zone = data.TimeZone(hours=0, minutes=0)
                     if (i_test_date.time_zone.hours >= 0 or
                             i_test_date.time_zone.minutes >= 0):
                         utc_offset = date_no_tz - i_test_date
@@ -1026,6 +1075,7 @@ class TestSuite(unittest.TestCase):
                         self.assertEqual(
                             duration, data.Duration(days=0),
                             i_test_date_str + " - " + j_test_date_str)
+        # TODO: test truncated TimePoints
 
     def test_timepoint_dumper(self):
         """Test the dumping of TimePoint instances."""
@@ -1058,15 +1108,13 @@ class TestSuite(unittest.TestCase):
                     num_expanded_year_digits=num_expanded_year_digits)
                 self.assertRaises(ctrl_exception, dumper.dump,
                                   ctrl_timepoint, format_)
-        value_error_timepoint = data.TimePoint(minute_of_hour=10)
-        value_error_timepoint.minute_of_hour = "1O"
-        self.assertRaises(ValueError, dumper.dump, value_error_timepoint, "%M")
 
     def test_timepoint_dumper_bounds_error_message(self):
         """Test the exception text contains the information expected"""
         the_error = TimePointDumperBoundsError("TimePoint1", "year",
                                                10, 20)
         the_string = the_error.__str__()
+        # FIXME:
         self.assertTrue("TimePoint1" in the_string,
                         "Failed to find TimePoint1 in {}".format(the_string))
         self.assertTrue("year" in the_string,
@@ -1102,7 +1150,7 @@ class TestSuite(unittest.TestCase):
         been copied, see issue #102 for more information"""
         time_point = data.TimePoint(year=2000, truncated=True,
                                     truncated_dump_format='CCYY')
-        the_copy = time_point.copy()
+        the_copy = time_point._copy()
         self.assertEqual(str(time_point), str(the_copy))
 
     def test_timepoint_parser(self):
@@ -1212,10 +1260,10 @@ class TestSuite(unittest.TestCase):
             minute_of_hour=ctrl_date.minute,
             second_of_minute=ctrl_date.second
         )
-        test_date.set_time_zone_to_utc()
+        # test_date = test_date.to_utc()
 
-        for test_date in [test_date, test_date.copy().to_week_date(),
-                          test_date.copy().to_ordinal_date()]:
+        for test_date in [test_date, test_date.to_week_date(),
+                          test_date.to_ordinal_date()]:
             # Test strftime (dumping):
             ctrl_data = ctrl_date.strftime(strftime_string)
             test_data = test_date.strftime(strftime_string)
@@ -1235,7 +1283,7 @@ class TestSuite(unittest.TestCase):
                         ctrl_dump, strptime_string)
                 test_dump = test_date.strftime(strptime_string)
                 test_data = parser.strptime(test_dump, strptime_string)
-                test_data.set_time_zone_to_utc()
+                test_data = test_data.to_utc()
 
                 self.assertEqual(test_dump, ctrl_dump, strptime_string)
 
@@ -1311,9 +1359,10 @@ class TestSuite(unittest.TestCase):
                     expression
                 )
             test_results = []
+            reps = test_recurrence.repetitions or 3
             for i, time_point in enumerate(test_recurrence):
-                if i > 2:
-                    break
+                if i >= reps:
+                    break  # Unbounded repetitions, just test 3 of them
                 test_results.append(str(time_point))
             self.assertEqual(test_results, ctrl_results, expression)
             if test_recurrence.start_point is None:
@@ -1323,17 +1372,17 @@ class TestSuite(unittest.TestCase):
                 forward_method = test_recurrence.get_next
                 backward_method = test_recurrence.get_prev
             test_points = [test_recurrence[0]]
-            test_points.append(forward_method(test_points[-1]))
-            test_points.append(forward_method(test_points[-1]))
+            for i in range(1, reps):
+                test_points.append(forward_method(test_points[-1]))
             test_results = [str(point) for point in test_points]
             self.assertEqual(test_results, ctrl_results, expression)
-            if test_recurrence[2] is not None:
-                test_points = [test_recurrence[2]]
+            # Test that going backwards beyond 1st point results in None:
+            test_points = [test_recurrence[reps - 1]]
+            for i in range(0, reps):
                 test_points.append(backward_method(test_points[-1]))
-                test_points.append(backward_method(test_points[-1]))
-                test_points.append(backward_method(test_points[-1]))
-            self.assertEqual(test_points[3], None, expression)
-            test_points.pop(3)
+            self.assertEqual(test_points[reps], None, expression)
+            # Test backwards method == reverse of forward:
+            test_points.pop(-1)
             test_points.reverse()
             test_results = [str(point) for point in test_points]
             self.assertEqual(test_results, ctrl_results, expression)
@@ -1352,6 +1401,21 @@ class TestSuite(unittest.TestCase):
                 test_is_member = test_recurrence.get_is_valid(timepoint)
                 self.assertEqual(test_is_member, ctrl_is_member,
                                  timepoint_expression + " in " + expression)
+        for expression, results in get_timerecurrence_first_after_tests():
+            try:
+                test_recurrence = parser.parse(expression)
+            except ISO8601SyntaxError:
+                raise ValueError(
+                    "TimeRecurrenceParser test failed to parse '%s'" %
+                    expression
+                )
+            for timepoint_expression, ctrl_result in results:
+                timepoint = parsers.parse_timepoint_expression(
+                    timepoint_expression)
+                test_result = str(test_recurrence.get_first_after(timepoint))
+                self.assertEqual(test_result, ctrl_result,
+                                 ctrl_result + " is first in " + expression
+                                 + " after " + timepoint_expression)
 
     def test_timerecurrence_parser(self):
         """Test the recurring date/time series parsing."""
@@ -1364,135 +1428,96 @@ class TestSuite(unittest.TestCase):
             ctrl_data = str(data.TimeRecurrence(**test_info))
             self.assertEqual(test_data, ctrl_data, expression)
 
-    # data provider for the test test_get_local_time_zone_no_dst
-    # the format for the parameters is
-    # [tz_seconds, expected_hours, expected_minutes]]
-    get_local_time_zone_no_dst = [
-        [45900, 12, 45],  # pacific/chatham, +12:45
-        [20700, 5, 45],  # asia/kathmandu, +05:45
-        [3600, 1, 0],  # arctic/longyearbyen, +01:00
-        [0, 0, 0],  # UTC
-        [-10800, -3, 0],  # america/sao_paulo, -03:00
-        [-12600, -3, 30]  # america/st_johns, -03:30
-    ]
+    def test_timerecurrence_comparison(self):
+        """Test the '==' operator and hash() on TimeRecurrences."""
+        parser = parsers.TimeRecurrenceParser()
+        tests = get_timerecurrence_comparison_tests()
+        for lhs_str, rhs_str, expected in tests:
+            lhs = parser.parse(lhs_str)
+            rhs = parser.parse(rhs_str)
+            test = lhs == rhs
+            assert test is expected, "{0} == {1}".format(lhs_str, rhs_str)
+            test = rhs == lhs
+            assert test is expected, "{0} == {1}".format(rhs_str, lhs_str)
+            test = lhs != rhs
+            assert test is not expected, "{0} != {1}".format(lhs_str, rhs_str)
+            test = hash(lhs) == hash(rhs)
+            assert test is expected, (
+                "hash of {0} == hash of {1}".format(lhs_str, rhs_str))
+            # If recurrences the same, list of timepoints must be equal:
+            if lhs.repetitions is not None and rhs.repetitions is not None:
+                # Note: don't list() unbounded recurrences!
+                test = list(lhs) == list(rhs)
+                assert test is expected
+        test_recurrence = parser.parse(tests[0][0])
+        for var in [7, 'foo', (1, 2), data.Duration(days=1)]:
+            self.assertFalse(test_recurrence == var)
 
-    @patch('metomi.isodatetime.timezone.time')
-    def test_get_local_time_zone_no_dst(self, mock_time):
-        """Test that the hour/minute returned is correct.
+    def test_timerecurrence_add(self):
+        """Test adding/subtracting Duration to/from TimeRecurrence"""
+        rep = 4
+        start_pt = data.TimePoint(year=2020, month_of_year=3, day_of_month=13)
+        second_pt = data.TimePoint(year=2020, month_of_year=3, day_of_month=20)
+        end_pt = data.TimePoint(year=2020, month_of_year=4, day_of_month=3)
+        dur = data.Duration(days=7)
+        offset = data.Duration(hours=9, minutes=59)
 
-        Parts of the time module are mocked so that we can specify scenarios
-        without daylight saving time."""
-        for tz_seconds, expected_hours, expected_minutes in \
-                self.get_local_time_zone_no_dst:
-            # for a pre-defined timezone
-            mock_time.timezone.__neg__.return_value = tz_seconds
-            # time without dst
-            mock_time.daylight = False
-            # and localtime also without dst
-            mock_localtime = Mock()
-            mock_time.localtime.return_value = mock_localtime
-            mock_localtime.tm_isdst = 0
-            hours, minutes = timezone.get_local_time_zone()
-            self.assertEqual(expected_hours, hours)
-            self.assertEqual(expected_minutes, minutes)
+        recurrence_fmt1 = data.TimeRecurrence(
+            repetitions=rep, start_point=start_pt, end_point=second_pt)
+        recurrence_fmt3 = data.TimeRecurrence(
+            repetitions=rep, start_point=start_pt, duration=dur)
+        recurrence_fmt4 = data.TimeRecurrence(
+            repetitions=rep, duration=dur, end_point=end_pt)
+        # Quick check, make sure these equivalent recurrences evaluate as equal
+        assert recurrence_fmt1 == recurrence_fmt3 == recurrence_fmt4
 
-    # data provider for the test test_get_local_time_zone_with_dst
-    # the format for the parameters is
-    # [tz_seconds, tz_alt_seconds, expected_hours, expected_minutes]
-    get_local_time_zone_with_dst = [
-        [45900, 49500, 13, 45],  # pacific/chatham, +12:45 and +13:45
-        [43200, 46800, 13, 0],  # pacific/auckland, +12:00 and +13:00
-        [3600, 7200, 2, 0],  # arctic/longyearbyen, +01:00
-        [0, 0, 0, 0],  # UTC
-        [-10800, -7200, -2, 0],  # america/sao_paulo, -03:00 and -02:00,
-        [-12600, -9000, -2, 30]  # america/st_johns, -03:30 and -02:30
-    ]
+        new_start_pt = start_pt + offset
+        expected = data.TimeRecurrence(
+            repetitions=rep, start_point=new_start_pt, duration=dur)
+        for recurrence in (recurrence_fmt1, recurrence_fmt3, recurrence_fmt4):
+            assert recurrence + offset == expected
+            assert offset + recurrence == expected
 
-    @patch('metomi.isodatetime.timezone.time')
-    def test_get_local_time_zone_with_dst(self, mock_time):
-        """Test that the hour/minute returned is correct
+        new_start_pt = start_pt - offset
+        expected = data.TimeRecurrence(
+            repetitions=rep, start_point=new_start_pt, duration=dur)
+        for recurrence in (recurrence_fmt1, recurrence_fmt3, recurrence_fmt4):
+            assert recurrence - offset == expected
 
-        Parts of the time module are mocked so that we can specify scenarios
-        with daylight saving time."""
-        for tz_seconds, tz_alt_seconds, expected_hours, expected_minutes in \
-                self.get_local_time_zone_with_dst:
-            # for a pre-defined timezone
-            mock_time.timezone.__neg__.return_value = tz_seconds
-            # time without dst
-            mock_time.daylight = True
-            # and localtime also without dst
-            mock_localtime = MagicMock()
-            mock_time.localtime.return_value = mock_localtime
-            mock_localtime.tm_isdst = 1
-            # and with the following alternative time for when dst is set
-            mock_time.altzone.__neg__.return_value = tz_alt_seconds
-            hours, minutes = timezone.get_local_time_zone()
-            self.assertEqual(expected_hours, hours)
-            self.assertEqual(expected_minutes, minutes)
+        with pytest.raises(TypeError):
+            recurrence_fmt3 + data.TimePoint(year=2049, month_of_year=2)
 
-    # data provider for the test test_get_local_time_zone_format
-    # the format for the parameters is
-    # [tz_seconds, tz_format_mode, expected_format]
-    get_test_get_local_time_zone_format = [
-        # UTC
-        # UTC, returns Z, flags are never used for UTC
-        [0, timezone.TimeZoneFormatMode.normal, "Z"],
-        # Positive values, some with minutes != 0
-        # asia/macao, +08:00
-        [28800, timezone.TimeZoneFormatMode.normal, "+0800"],
-        # asia/macao, +08:00
-        [28800, timezone.TimeZoneFormatMode.extended, "+08:00"],
-        # asia/macao, +08:00
-        [28800, timezone.TimeZoneFormatMode.reduced, "+08"],
-        # pacific/chatham, +12:45
-        [45900, timezone.TimeZoneFormatMode.normal, "+1245"],
-        # pacific/chatham, +12:45
-        [45900, timezone.TimeZoneFormatMode.extended, "+12:45"],
-        # pacific/chatham, +12:45
-        [45900, timezone.TimeZoneFormatMode.reduced, "+1245"],
-        # Negative values, some with minutes != 0
-        # america/buenos_aires, -03:00
-        [-10800, timezone.TimeZoneFormatMode.normal, "-0300"],
-        # america/buenos_aires, -03:00
-        [-10800, timezone.TimeZoneFormatMode.extended, "-03:00"],
-        # america/buenos_aires, -03:00
-        [-10800, timezone.TimeZoneFormatMode.reduced, "-03"],
-        # america/st_johns, -03:30
-        [-12600, timezone.TimeZoneFormatMode.normal, "-0330"],
-        # america/st_johns, -03:30
-        [-12600, timezone.TimeZoneFormatMode.extended, "-03:30"],
-        # america/st_johns, -03:30
-        [-12600, timezone.TimeZoneFormatMode.reduced, "-0330"]
-    ]
-
-    @patch('metomi.isodatetime.timezone.time')
-    def test_get_local_time_zone_format(self, mock_time):
-        """Test that the UTC offset string format is correct
-
-        Parts of the time module are mocked so that we can specify scenarios
-        with certain timezone seconds offsets. DST is not really
-        important for this test case"""
-        for tz_seconds, tz_format_mode, expected_format in \
-                self.get_test_get_local_time_zone_format:
-            # for a pre-defined timezone
-            mock_time.timezone.__neg__.return_value = tz_seconds
-            # time without dst
-            mock_time.daylight = False
-            # and localtime also without dst
-            mock_localtime = Mock()
-            mock_time.localtime.return_value = mock_localtime
-            mock_localtime.tm_isdst = 0
-            tz_format = timezone.get_local_time_zone_format(tz_format_mode)
-            self.assertEqual(expected_format, tz_format)
+    def test_invalid_timerecurrence(self):
+        """Test that init'ing a TimeRecurrence with bad inputs raises error."""
+        start_pt = data.TimePoint(year=2020, day_of_month=9)
+        end_pt = data.TimePoint(year=2020, day_of_month=30)
+        dur = data.Duration(hours=36)
+        _tests = [{"repetitions": 0, "start_point": start_pt, "duration": dur}]
+        for reps in (4, None):
+            _tests += [
+                {"repetitions": reps, "start_point": start_pt,
+                 "end_point": end_pt, "duration": dur},
+                {"repetitions": reps, "start_point": start_pt},
+                {"repetitions": reps, "end_point": end_pt},
+                {"repetitions": reps, "duration": dur},
+                {"repetitions": reps, "start_point": start_pt,
+                 "duration": data.Duration(minutes=-20)},
+                {"repetitions": reps, "start_point": end_pt,
+                 "end_point": start_pt}
+            ]
+        for kwargs in _tests:
+            with pytest.raises(BadInputError):
+                data.TimeRecurrence(**kwargs)
 
     def test_timepoint_dump_format(self):
         """Test the timepoint format dump when values are programmatically
         set to None"""
+        # TODO: Get rid of this now that TimePoint is immutable?
         t = data.TimePoint(year="1984")
         # commenting out month_of_year here is enough to make the test pass
-        t.month_of_year = None
-        t.day_of_year = None
-        t.week_of_year = None
+        t._month_of_year = None
+        t._day_of_year = None
+        t._week_of_year = None
         with self.assertRaises(RuntimeError):
             self.assertEqual("1984-01-01T00:00:00Z", str(t))
         # QUESTION: What was this test meant to do exactly?
